/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { TextureSpec, Values, UniformSpec, DefineSpec } from '../../mol-gl/renderable/schema';
import { WebGLContext } from '../../mol-gl/webgl/context';
import { ComputeRenderable } from '../../mol-gl/renderable';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { RenderTarget } from '../../mol-gl/webgl/render-target';
import { DrawPass } from './draw';
import { Camera, ICamera } from '../../mol-canvas3d/camera';
import { StereoCamera } from '../camera/stereo';
declare const PostprocessingSchema: {
    tColor: TextureSpec<"texture">;
    tPackedDepth: TextureSpec<"texture">;
    uTexSize: UniformSpec<"v2">;
    dOrthographic: DefineSpec<"number">;
    uNear: UniformSpec<"f">;
    uFar: UniformSpec<"f">;
    uFogNear: UniformSpec<"f">;
    uFogFar: UniformSpec<"f">;
    uFogColor: UniformSpec<"v3">;
    dOcclusionEnable: DefineSpec<"boolean">;
    dOcclusionKernelSize: DefineSpec<"number">;
    uOcclusionBias: UniformSpec<"f">;
    uOcclusionRadius: UniformSpec<"f">;
    dOutlineEnable: DefineSpec<"boolean">;
    uOutlineScale: UniformSpec<"f">;
    uOutlineThreshold: UniformSpec<"f">;
    drawCount: import("../../mol-gl/renderable/schema").ValueSpec<"number">;
    instanceCount: import("../../mol-gl/renderable/schema").ValueSpec<"number">;
    aPosition: import("../../mol-gl/renderable/schema").AttributeSpec<"float32">;
    uQuadScale: UniformSpec<"v2">;
};
declare type PostprocessingRenderable = ComputeRenderable<Values<typeof PostprocessingSchema>>;
export declare const PostprocessingParams: {
    occlusion: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "off"> | PD.NamedParams<PD.Normalize<{
        kernelSize: number;
        bias: number;
        radius: number;
    }>, "on">>;
    outline: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "off"> | PD.NamedParams<PD.Normalize<{
        scale: number;
        threshold: number;
    }>, "on">>;
};
export declare type PostprocessingProps = PD.Values<typeof PostprocessingParams>;
export declare class PostprocessingPass {
    private webgl;
    private drawPass;
    static isEnabled(props: PostprocessingProps): boolean;
    target: RenderTarget;
    renderable: PostprocessingRenderable;
    constructor(webgl: WebGLContext, drawPass: DrawPass);
    private bindTarget;
    syncSize(): void;
    _render(camera: ICamera, toDrawingBuffer: boolean, props: PostprocessingProps): void;
    render(camera: Camera | StereoCamera, toDrawingBuffer: boolean, props: PostprocessingProps): void;
}
export {};
