/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { __assign, __awaiter, __generator } from "tslib";
import { CreateOrbitalRepresentation3D, CreateOrbitalVolume, StaticBasisAndOrbitals } from '../../extensions/alpha-orbitals/transforms';
import { createPluginAsync, DefaultPluginSpec } from '../../mol-plugin';
import { PluginConfig } from '../../mol-plugin/config';
import { ColorNames } from '../../mol-util/color/names';
import { ParamDefinition } from '../../mol-util/param-definition';
import { mountControls } from './controls';
import { DemoMoleculeSDF, DemoOrbitals } from './example-data';
import { BehaviorSubject } from 'rxjs';
import { debounceTime, skip } from 'rxjs/operators';
import './index.html';
require('mol-plugin-ui/skin/light.scss');
var AlphaOrbitalsExample = /** @class */ (function () {
    function AlphaOrbitalsExample() {
        this.params = new BehaviorSubject({});
        this.state = new BehaviorSubject({ orbitalIndex: 32, isoValue: 1, gpuSurface: false, cpuCompute: false });
        this.currentParams = __assign({}, this.state.value);
    }
    AlphaOrbitalsExample.prototype.init = function (target) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, createPluginAsync(typeof target === 'string' ? document.getElementById(target) : target, __assign(__assign({}, DefaultPluginSpec), { layout: {
                                    initial: {
                                        isExpanded: false,
                                        showControls: false
                                    },
                                    controls: { left: 'none', right: 'none', top: 'none', bottom: 'none' },
                                }, config: [
                                    [PluginConfig.Viewport.ShowExpand, false],
                                    [PluginConfig.Viewport.ShowControls, false],
                                    [PluginConfig.Viewport.ShowSelectionMode, false],
                                    [PluginConfig.Viewport.ShowAnimation, false],
                                ] }))];
                    case 1:
                        _a.plugin = _b.sent();
                        this.plugin.managers.interactivity.setProps({ granularity: 'element' });
                        this.load(__assign({ moleculeSdf: DemoMoleculeSDF }, DemoOrbitals));
                        mountControls(this, document.getElementById('controls'));
                        return [2 /*return*/];
                }
            });
        });
    };
    AlphaOrbitalsExample.prototype.setIndex = function () {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            var state;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!((_a = this.volume) === null || _a === void 0 ? void 0 : _a.isOk))
                            return [2 /*return*/];
                        state = this.state.value;
                        return [4 /*yield*/, this.plugin.build().to(this.volume).update(CreateOrbitalVolume, function () { return ({ index: state.orbitalIndex, cpuCompute: state.cpuCompute }); }).commit()];
                    case 1:
                        _b.sent();
                        if (!(this.currentParams.gpuSurface !== this.state.value.gpuSurface)) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.setIsovalue()];
                    case 2:
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        this.currentParams = this.state.value;
                        return [2 /*return*/];
                }
            });
        });
    };
    AlphaOrbitalsExample.prototype.setIsovalue = function () {
        this.currentParams = this.state.value;
        var update = this.plugin.build();
        update.to(this.positive).update(this.volumeParams('positive', ColorNames.blue));
        update.to(this.negative).update(this.volumeParams('negative', ColorNames.red));
        return update.commit();
    };
    AlphaOrbitalsExample.prototype.volumeParams = function (kind, color) {
        return {
            alpha: 1.0,
            color: color,
            directVolume: this.state.value.gpuSurface,
            kind: kind,
            relativeIsovalue: this.state.value.isoValue,
            pickable: false,
            xrayShaded: false
        };
    };
    AlphaOrbitalsExample.prototype.load = function (input) {
        return __awaiter(this, void 0, void 0, function () {
            var data, trajectory, model, structure, all, state, _a, repr;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.plugin.clear()];
                    case 1:
                        _b.sent();
                        return [4 /*yield*/, this.plugin.builders.data.rawData({ data: input.moleculeSdf }, { state: { isGhost: true } })];
                    case 2:
                        data = _b.sent();
                        return [4 /*yield*/, this.plugin.builders.structure.parseTrajectory(data, 'mol')];
                    case 3:
                        trajectory = _b.sent();
                        return [4 /*yield*/, this.plugin.builders.structure.createModel(trajectory)];
                    case 4:
                        model = _b.sent();
                        return [4 /*yield*/, this.plugin.builders.structure.createStructure(model)];
                    case 5:
                        structure = _b.sent();
                        return [4 /*yield*/, this.plugin.builders.structure.tryCreateComponentStatic(structure, 'all')];
                    case 6:
                        all = _b.sent();
                        if (!all) return [3 /*break*/, 8];
                        return [4 /*yield*/, this.plugin.builders.structure.representation.addRepresentation(all, { type: 'ball-and-stick', color: 'element-symbol', colorParams: { carbonColor: { name: 'element-symbol', params: {} } } })];
                    case 7:
                        _b.sent();
                        _b.label = 8;
                    case 8:
                        state = this.state.value;
                        _a = this;
                        return [4 /*yield*/, this.plugin.build().toRoot()
                                .apply(StaticBasisAndOrbitals, { basis: input.basis, order: input.order, orbitals: input.orbitals })
                                .apply(CreateOrbitalVolume, { index: state.orbitalIndex, forceCpuCompute: this.currentParams.cpuCompute })
                                .commit()];
                    case 9:
                        _a.volume = _b.sent();
                        if (!this.volume.isOk) {
                            this.volume = void 0;
                            return [2 /*return*/];
                        }
                        repr = this.plugin.build().to(this.volume);
                        this.positive = repr.apply(CreateOrbitalRepresentation3D, this.volumeParams('positive', ColorNames.blue)).selector;
                        this.negative = repr.apply(CreateOrbitalRepresentation3D, this.volumeParams('negative', ColorNames.red)).selector;
                        return [4 /*yield*/, repr.commit()];
                    case 10:
                        _b.sent();
                        this.params.next({
                            orbitalIndex: ParamDefinition.Numeric(this.currentParams.orbitalIndex, { min: 0, max: input.orbitals.length - 1 }, { immediateUpdate: true, isEssential: true }),
                            isoValue: ParamDefinition.Numeric(this.currentParams.isoValue, { min: 0.5, max: 3, step: 0.1 }, { immediateUpdate: true, isEssential: false }),
                            gpuSurface: ParamDefinition.Boolean(this.currentParams.gpuSurface),
                            cpuCompute: ParamDefinition.Boolean(this.currentParams.cpuCompute, { label: 'CPU Compute' })
                        });
                        this.state.pipe(skip(1), debounceTime(1000 / 24)).subscribe(function (params) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                if (params.orbitalIndex !== this.currentParams.orbitalIndex
                                    || params.cpuCompute !== this.currentParams.cpuCompute) {
                                    this.setIndex();
                                }
                                else if (params.isoValue !== this.currentParams.isoValue || params.gpuSurface !== this.currentParams.gpuSurface) {
                                    this.setIsovalue();
                                }
                                return [2 /*return*/];
                            });
                        }); });
                        return [2 /*return*/];
                }
            });
        });
    };
    return AlphaOrbitalsExample;
}());
export { AlphaOrbitalsExample };
window.AlphaOrbitalsExample = new AlphaOrbitalsExample();
//# sourceMappingURL=index.js.map