/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __assign } from "tslib";
import { createComputeRenderable } from '../../renderable';
import { createComputeRenderItem } from '../../webgl/render-item';
import { TextureSpec } from '../../renderable/schema';
import { ShaderCode } from '../../../mol-gl/shader-code';
import { ValueCell } from '../../../mol-util';
import { decodeFloatRGB } from '../../../mol-util/float-packing';
import { QuadSchema, QuadValues } from '../util';
import quad_vert from '../../../mol-gl/shader/quad.vert';
import sum_frag from '../../../mol-gl/shader/histogram-pyramid/sum.frag';
var HistopyramidSumSchema = __assign(__assign({}, QuadSchema), { tTexture: TextureSpec('texture', 'rgba', 'float', 'nearest') });
var HistopyramidSumRenderable;
function getHistopyramidSumRenderable(ctx, texture) {
    if (HistopyramidSumRenderable) {
        ValueCell.update(HistopyramidSumRenderable.values.tTexture, texture);
        HistopyramidSumRenderable.update();
        return HistopyramidSumRenderable;
    }
    else {
        var values = __assign(__assign({}, QuadValues), { tTexture: ValueCell.create(texture) });
        var schema = __assign({}, HistopyramidSumSchema);
        var shaderCode = ShaderCode('sum', quad_vert, sum_frag);
        var renderItem = createComputeRenderItem(ctx, 'triangles', shaderCode, schema, values);
        HistopyramidSumRenderable = createComputeRenderable(renderItem, values);
        return HistopyramidSumRenderable;
    }
}
var SumTexture;
function getSumTexture(ctx) {
    if (SumTexture)
        return SumTexture;
    SumTexture = ctx.resources.texture('image-uint8', 'rgba', 'ubyte', 'nearest');
    SumTexture.define(1, 1);
    return SumTexture;
}
function setRenderingDefaults(ctx) {
    var gl = ctx.gl, state = ctx.state;
    state.disable(gl.CULL_FACE);
    state.disable(gl.BLEND);
    state.disable(gl.DEPTH_TEST);
    state.disable(gl.SCISSOR_TEST);
    state.depthMask(false);
    state.colorMask(true, true, true, true);
    state.clearColor(0, 0, 0, 0);
}
var sumArray = new Uint8Array(4);
export function getHistopyramidSum(ctx, pyramidTopTexture) {
    var gl = ctx.gl, resources = ctx.resources;
    var renderable = getHistopyramidSumRenderable(ctx, pyramidTopTexture);
    ctx.state.currentRenderItemId = -1;
    var framebuffer = resources.framebuffer();
    var sumTexture = getSumTexture(ctx);
    sumTexture.attachFramebuffer(framebuffer, 0);
    setRenderingDefaults(ctx);
    gl.viewport(0, 0, 1, 1);
    renderable.render();
    gl.finish();
    ctx.readPixels(0, 0, 1, 1, sumArray);
    ctx.unbindFramebuffer();
    return decodeFloatRGB(sumArray[0], sumArray[1], sumArray[2]);
}
//# sourceMappingURL=sum.js.map